
/*
 *	nlist - get type and value entries from a name list.
 *
 *	Copyright (C) Microsoft Corporation, 1982
 *
 *    N.B.: For 8086 abs symbols, the segment and offset are
 *	combined.  Any overflow is masked off.  It is possible
 *	to obtain bad values without any error indication when
 *	used on large model 8086.
 */

#include <stdio.h>
#include <a.out.h>
#include <sys/relsym.h>


#ifdef M_WORDSWAP
#  define hostwswap 1
#else
#  define hostwswap 0
#endif




nlist(name, list)
char *name;
struct nlist *list;
{
	register struct nlist *lp;
	register int i;
	register char *cp;
	register FILE *fp;
	register long syms;
	register char stype = XR_SXOUT;
	struct xexec xexec;
	struct bexec *bp;
	struct aexec *ap;
	unsigned short magic;
	struct sym sym;
	struct bsym bsym;
	struct asym asym;
	int overflow = 0;
	char symbuf[SYMLENGTH];
	int hsize;

	for (lp = list; lp->n_name[0]; lp++) {
		lp->n_type = 0;
		lp->n_value = 0;
	}
	if ((fp = fopen(name, "r")) == NULL) {
		return(-1);
	}
	hsize = fread((char *) &xexec, sizeof(char), sizeof(xexec), fp);
	if ((magic = xexec.x_magic) == 0) {
		if (hsize < sizeof(struct bexec)) {
			goto bad;
		}
		bp = (struct bexec *) &xexec;
		magic = (int) bp->xb_magic;
		stype = XR_SBOUT;
	}
	switch (magic) {

	case X_MAGIC:
		if (hsize < sizeof(struct xexec)) {
			goto bad;
		}
/*		if (((xexec.x_cpu & XC_WSWAP)? 1 : 0) != hostwswap) {*/
/*			goto bad;*/
/*		}*/
		stype = xexec.x_relsym & XR_SYM;
		fseek(fp, XSYMPOS(&xexec), 0);
		syms = xexec.x_syms;
		break;

	case FMAGIC:
	case NMAGIC:
	case IMAGIC:
	case OMAGIC:
		if (stype == XR_SBOUT) {
			fseek(fp, BSYMPOS(bp), 0);
			syms = bp->xb_syms;
		} else {
			if (hsize < sizeof(struct aexec)) {
				goto bad;
			}
			stype = XR_SAOUT;
			ap = (struct aexec *) &xexec;
			fseek(fp, ASYMPOS(ap), 0);
			syms = ap->xa_syms;
		}
		break;

	default: goto bad;
	}
	while (syms) {
		if (stype == XR_SAOUT) {
			fread((char *) &asym, sizeof(asym), 1, fp);
			syms -= sizeof(asym);
			if (feof(fp)) {
				goto bad;
			}
			sym.s_type = asym.sa_type;
			sym.s_value = asym.sa_value;
			cp = asym.sa_name;
		} else {
			switch (stype) {

			case XR_SBOUT:
				fread((char *)&bsym, sizeof(bsym),1,fp);
				syms -= sizeof(bsym);
				sym.s_type = bsym.sb_type;
				sym.s_value = bsym.sb_value;
				break;

			case XR_S86ABS:
				fread((char *)&sym, sizeof(sym), 1, fp);
				syms -= sizeof(sym);
				sym.s_value &= 0x0000ffffL;
				break;

			case XR_SXOUT:
				fread((char *)&sym, sizeof(sym), 1, fp);
				syms -= sizeof(sym);
				break;

			default: goto bad;
			}
			if (feof(fp)) {
				goto bad;
			}
			cp = symbuf;
			do {
				if ((cp >= &symbuf[SYMLENGTH]) 
				    || (i = getc(fp)) == EOF) {
					goto bad;
				}
			} while (*cp++ = i);
			syms -= strlen(symbuf) + 1;
			cp = symbuf;
		}
		for (lp = list; lp->n_name[0]; lp++) {
			for (i = 0; i < 8; i++) {
				if (lp->n_name[i] != cp[i]) {
					goto nomatch;
				}
				if (cp[i] == 0) {
					break;
				}
			}
			if ((sizeof(long) > sizeof(int)) &&
			    (sym.s_value & 0xffff0000L)) {
				overflow = -1;
				break;
			}
			lp->n_value = (int) sym.s_value;
			lp->n_type = (char) sym.s_type;
			break;
		nomatch:	;
		}
	}
	fclose(fp);
	return(overflow);

bad:	fclose(fp);
	return(-1);
}
